/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2014 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.adobe.cq.social.srp.internal;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.StringUtils;
import org.apache.sling.api.SlingConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adobe.cq.social.srp.utilities.internal.InternalSocialResourceUtilities;
import com.adobe.granite.activitystreams.JsonConstants;

/**
 * This class maps from SoCo keys and values to those used by the common schema developed by Adobe Cloud Storage. This
 * class is only intended to be used by Communities code.
 */
public abstract class AbstractSchemaMapper {
    /* ---------------- Stored Document fields ---------------- */
    /** the different descriptions that can be possible. */
    public static final String AS_ACCEPTFILETYPES = "acceptFileTypes";
    public static final String AS_ALLOWFILEUPLOADS = "allowFileUploads";
    public static final String AS_ALLOW_REPLIES = "allowRepliesToComments";
    public static final String AS_APPROVED = "approved_b";
    public static final String AS_AUTHOR = "author_username";
    public static final String AS_AUTHORIZABLE_ID = "authorizableId_t";
    public static final String AS_AUTHOR_DISPLAY_NAME = "author_display_name";
    public static final String AS_AUTHOR_DISPLAY_NAME_CI = "author_display_name_ci";
    public static final String AS_AUTHOR_PROFILE_URL = "author_profile_url";
    public static final String AS_AUTHOR_CI = "author_username_ci";
    public static final String AS_AUTHOR_IMAGE_URL = "author_image_url";
    public static final String AS_BASE_TYPE = "base_type_s";
    public static final String AS_IS_CLOSED = "is_closed_b";
    public static final String AS_COMPANY_ID = "company_db_id";
    public static final String AS_COMPOSED_BY = "composedBy_t";
    public static final String AS_CQ_DATA = "cqdata";
    public static final String AS_CQTAGS = "cqtags_ss";
    public static final String AS_CREATED = "timestamp";
    public static final String AS_ENTITY_URL = "entity_url";
    public static final String AS_FEATURES = "features";
    public static final String AS_ID = "_id"; // OPADS used to have this as id
    public static final String AS_IS_DRAFT = "is_draft_b";
    public static final String AS_IS_FLAGGED = "is_flagged_b";
    public static final String AS_IS_FLAGGED_HIDDEN = "is_flaggedHidden_b";
    public static final String AS_IS_REPLY = "is_reply_b";
    public static final String AS_MAXFILESIZE = "maxFileSize";
    public static final String AS_MAXIMAGEFILESIZE = "maxImageFileSize";
    public static final String AS_MODERATE_COMMENTS = "moderateComments";
    public static final String AS_PARENT_ID = "parent_id_s";
    public static final String AS_PINNED = "pinned_b";
    public static final String AS_PROVIDER_ID = "provider_id";
    public static final String AS_PUBLISH_DATE = "publishDate_dt";
    public static final String AS_PUBLISH_JOB_ID = "publishJobId_s";
    public static final String AS_REPORT_SUITE = "report_suite";
    public static final String AS_REQUIRELOGIN = "requireLogin";
    public static final String AS_RESOURCE_TYPE = "resource_type_s";
    public static final String AS_ROOT_COMMENT_SYSTEM = "thread_id_s"; // OPADS used to have rootcommentsystem_s
    public static final String AS_RTEENABLED = "rteEnabled";
    public static final String AS_TALLY_RESPONSE = "response_s";
    public static final String AS_TITLE = "title_t";
    public static final String AS_VERBATIM = "verbatim";
    public static final String AS_VERBATIM_LANGUAGE_INDEX = "verbatim_";
    public static final String AS_ATTACHMENT_LENGTH = "length";
    public static final String AS_UPLOADDATE = "uploadDate";
    public static final String AS_ADDRESS = "address_s";

    // Unique solr 'id' required to be present
    public static final String SOLR_ID = "id";

    /*
     * Even though the sentiment field name is the same in AS and SoCo, we have to define it so that it doesn't go in
     * cq_data
     */
    public static final String AS_SENTIMENT = "sentiment";

    /* ---------------- Stored attachment fields ---------------- */
    public static final String AS_CONTENT_TYPE = "content-type";
    public static final String AS_ATTACHMENT = "attachment";

    /* ---------------- Activity Streams searchable fields ---------------- */
    public static final String AS_VERB = "verb_s";
    public static final String AS_ACTOR_ID = "actorid_s";

    /* ---------------- SoCo doc fields ---------------- */
    private static final String CQ_ACCEPTFILETYPES = "acceptFileTypes";
    public static final String CQ_ADDED = InternalSocialResourceUtilities.PN_DATE;
    public static final String CQ_ALLOWFILEUPLOADS = "allowFileUploads";
    public static final String CQ_ALLOW_REPLIES = "allowRepliesToComments";
    public static final String CQ_APPROVED = "approved";
    public static final String CQ_AUTHORIZABLE_ID = "authorizableId";
    public static final String CQ_AUTHOR_IMAGE_URL = "author_image_url";
    public static final String CQ_AUTHOR_PROFILE_URL = "author_profile_url";
    public static final String CQ_AUTHOR_DISPLAY_NAME = "author_display_name";
    public static final String CQ_AUTHOR_EMAIL = "author_display_name";
    public static final String CQ_BASE_TYPE = InternalSocialResourceUtilities.PN_BASETYPE;
    public static final String CQ_COMPOSED_BY = "composedBy";
    public static final String CQ_DRAFT = "isDraft";
    public static final String CQ_IS_CLOSED = "isClosed";
    public static final String CQ_ENTITY_URL = "entity_url";
    public static final String CQ_IS_FLAGGED = "isFlagged";
    public static final String CQ_FLAGGED_HIDDEN = "isFlaggedHidden";
    public static final String CQ_IS_REPLY = InternalSocialResourceUtilities.PN_IS_REPLY;
    public static final String CQ_KEY = InternalSocialResourceUtilities.PN_DS_KEY;
    public static final String CQ_MAXFILESIZE = "maxFileSize";
    public static final String CQ_MAXIMAGEFILESIZE = "maxImageFileSize";
    public static final String CQ_MODERATE_COMMENTS = "moderateComments";
    public static final String CQ_PARENT_ID = InternalSocialResourceUtilities.PN_PARENTID;
    public static final String CQ_PINNED = "pinned";
    public static final String CQ_PUBLISH_DATE = "publishDate";
    public static final String CQ_PUBLISH_JOB_ID = "publishJobId";
    public static final String CQ_REQUIRELOGIN = "requireLogin";
    public static final String CQ_RESOURCE_TYPE = SlingConstants.NAMESPACE_PREFIX + ":"
            + SlingConstants.PROPERTY_RESOURCE_TYPE;
    public static final String CQ_ROOT_COMMENT_SYSTEM = InternalSocialResourceUtilities.PN_CS_ROOT;
    public static final String CQ_RTEENABLED = "rteEnabled";
    public static final String CQ_SPAM = "isSpam";
    public static final String CQ_TAGS = "cq:tags";
    public static final String CQ_TALLY_RESPONSE = "response";
    public static final String CQ_TALLY_TIMESTAMP = "timestamp";
    public static final String CQ_TITLE = "jcr:title";
    public static final String CQ_USER_IDENTIFIER = "userIdentifier";
    public static final String CQ_DESCRIPTION = "jcr:description";
    public static final String CQ_LAST_MODIFIED = "cq:lastModified";
    public static final String CQ_SENTIMENT = "sentiment";

    /* ---------------- SoCo attachment fields ---------------- */
    public static final String CQ_ATTACHMENT = "nt:file";
    public static final String CQ_CONTENT_TYPE = "mimetype";

    /**
     * Search collections has its own mapping for some fields.
     */
    public static final String CQ_SEARCH_PATH = ":path"; // path filter from isearch collections
    public static final String CQ_SEARCH_TAG_NAME = "cq:tags";  // tag in lucene query string from search collections
    public static final String CQ_SEARCH_PARENT_ID = ":parent"; // path filter from isearch collections

    /*
     * Machine Translation Fields
     */
    public static final String CQ_MT_TRANS_DATE = "translationDate";
    public static final String CQ_MT_TRANSLATION = "translation";
    protected static final String CQ_MT_LANGAUGE = "mtlanguage";

    /*
     * Activity Streams fields
     */
    public static final String CQ_AS_VERB = JsonConstants.PROPERTY_VERB;
    public static final String CQ_AS_ACTOR_ID = JsonConstants.PROPERTY_ACTOR_ID;

    /* suffix pattern to identify searchable keys, as defined in the solr schema_soco.xml config file */
    private static final String SEARCHABLE_SUFFIX =
        "s|ss|t|ts|b|bs|i|is|tl|tls|f|fs|d|ds|dt|dts|tf|tfs|td|tds|tdt|tdts|latlong|latlongs|tg|tgs|en|ens|ja|jas";
    /*
     * This is the regular exp. that determines if a key (name) is searchable or not. [^\\s] - must start with one or
     * more characters except white space \\_ - follow by a underscore '_' ?i - ignore case in match $ - end of the
     * string
     */
    private static final String SUFFIX_PATTERN = "([^\\s]+(\\_(?i)(" + SEARCHABLE_SUFFIX + "))$)";
    private static final Pattern PATTERN = Pattern.compile(SUFFIX_PATTERN);

    private static final String DATE_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";
    private static final String CQ_DATE_FORMAT = "EEE MMM dd HH:mm:ss zzz yyyy";

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractSchemaMapper.class);

    // These are the sets of attachment related strings that need explicit mapping. Everything else is blindly
    // copied to the cqdata subhash (if not a searchable key) or to the toplevel hash (if a searchable key)..
    private final Set<String> CQ_ATTACHMENT_KEYS;
    private final Set<String> AS_ATTACHMENT_KEYS;

    private final Map<String, String> TOP_LEVEL_TO_SCHEMA_KEYS;
    private final Map<String, String> TOP_LEVEL_TO_CQ_KEYS;
    private final Map<String, String> TRANSLATION_TO_SCHEMA_KEYS;

    // Don't want to maintain it in two places. Using dates would be nice, but AS JSOn would be a pain.
    private static final List<String> SUBHASH_DATE_FIELDS = Arrays.asList(CQ_LAST_MODIFIED, CQ_TALLY_TIMESTAMP,
        CQ_MT_TRANS_DATE, "jcr:created", "jcr:lastModified");

    /**
     * Prefix for non dynamic dates so we can convert from the long format back to calendar.
     */
    public static final String AS_CQ_SUBHASH_DATE = "SRP_CQ_SUBHASH_123098_DATE_";

    /** ctor. */
    public AbstractSchemaMapper() {

        TOP_LEVEL_TO_CQ_KEYS = new HashMap<String, String>();
        TOP_LEVEL_TO_CQ_KEYS.put(AS_APPROVED, CQ_APPROVED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_CQTAGS, CQ_TAGS);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_CREATED, CQ_ADDED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_IS_DRAFT, CQ_DRAFT);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_PINNED, CQ_PINNED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_RESOURCE_TYPE, CQ_RESOURCE_TYPE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_TITLE, CQ_TITLE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_PARENT_ID, CQ_PARENT_ID);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_PROVIDER_ID, CQ_KEY);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_AUTHOR, CQ_USER_IDENTIFIER);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_AUTHORIZABLE_ID, CQ_AUTHORIZABLE_ID);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_COMPOSED_BY, CQ_COMPOSED_BY);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ROOT_COMMENT_SYSTEM, CQ_ROOT_COMMENT_SYSTEM);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_TALLY_RESPONSE, CQ_TALLY_RESPONSE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_AUTHOR_IMAGE_URL, CQ_AUTHOR_IMAGE_URL);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_AUTHOR_PROFILE_URL, CQ_AUTHOR_PROFILE_URL);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_AUTHOR_DISPLAY_NAME, CQ_AUTHOR_DISPLAY_NAME);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_BASE_TYPE, CQ_BASE_TYPE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ENTITY_URL, CQ_ENTITY_URL);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_IS_REPLY, CQ_IS_REPLY);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_MODERATE_COMMENTS, CQ_MODERATE_COMMENTS);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ALLOW_REPLIES, CQ_ALLOW_REPLIES);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_IS_CLOSED, CQ_IS_CLOSED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_IS_FLAGGED, CQ_IS_FLAGGED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_IS_FLAGGED_HIDDEN, CQ_FLAGGED_HIDDEN);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ALLOWFILEUPLOADS, CQ_ALLOWFILEUPLOADS);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_MAXFILESIZE, CQ_MAXFILESIZE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ACCEPTFILETYPES, CQ_ACCEPTFILETYPES);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_RTEENABLED, CQ_RTEENABLED);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_REQUIRELOGIN, CQ_REQUIRELOGIN);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_MAXIMAGEFILESIZE, CQ_MAXIMAGEFILESIZE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_PUBLISH_DATE, CQ_PUBLISH_DATE);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_PUBLISH_JOB_ID, CQ_PUBLISH_JOB_ID);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_ACTOR_ID, CQ_AS_ACTOR_ID);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_VERB, CQ_AS_VERB);
        TOP_LEVEL_TO_CQ_KEYS.put(AS_SENTIMENT, CQ_SENTIMENT);

        final Map<String, String> mapperKeys = getMapperSpecificKeys();
        TOP_LEVEL_TO_CQ_KEYS.putAll(mapperKeys);

        TOP_LEVEL_TO_SCHEMA_KEYS = new HashMap<String, String>(TOP_LEVEL_TO_CQ_KEYS.size());
        for (final Entry<String, String> entry : TOP_LEVEL_TO_CQ_KEYS.entrySet()) {
            TOP_LEVEL_TO_SCHEMA_KEYS.put(entry.getValue(), entry.getKey());
        }

        AS_ATTACHMENT_KEYS = new HashSet<String>();
        AS_ATTACHMENT_KEYS.add(AS_CONTENT_TYPE);
        AS_ATTACHMENT_KEYS.add(AS_ATTACHMENT);
        AS_ATTACHMENT_KEYS.add(AS_PROVIDER_ID);
        AS_ATTACHMENT_KEYS.add(AS_UPLOADDATE);

        CQ_ATTACHMENT_KEYS = new HashSet<String>();
        CQ_ATTACHMENT_KEYS.add(CQ_CONTENT_TYPE);
        CQ_ATTACHMENT_KEYS.add(CQ_ATTACHMENT);

        TRANSLATION_TO_SCHEMA_KEYS = new HashMap<String, String>();
        updateTranslationSchemaKeys(TRANSLATION_TO_SCHEMA_KEYS);
    }

    /**
     * Update translation schema keys
     * @param translationSchmeKeys Translation Schema Keys
     */
    protected void updateTranslationSchemaKeys(final Map<String, String> translationSchmeKeys) {
	}

    /**
     * fetch the keys that are specific to this impl that need to be mapped.
     * @return the keys (going from the schema key to the soco key) to be mapped
     */
    public abstract Map<String, String> getMapperSpecificKeys() {
		return null;
	}

    /** @return the report suite used in this mapper. */
    public abstract String getReportSuite() {
		return null;
	}

    /**
     * set the report suite used in this mapper.
     * @param reportSuite the report suite
     */
    public abstract void setReportSuite(String reportSuite) {
	}

    /**
     * @return the key used in the db schema that tells the description
     */
    public static String getSchemaDescriptionKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the resource type.
     */
    public static String getSchemaResourceTypeKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the location of the root comment system.
     */
    public static String getSchemaRootCommentSystemKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell the author's display name.
     */
    public static String getSocoAuthorDisplayNameKey() {
		return null;
	}

    /**
     * @return the key used in the soco to tell the author's avatar url.
     */
    public static String getSocoAuthorImageUrlKey() {
		return null;
	}

    /**
     * @return the key used in the soco to tell the author's avatar url.
     */
    public static String getSocoAuthorProfileUrlKey() {
		return null;
	}

    /**
     * @return the key used in the soco to tell the entity url.
     */
    public static String getSocoEntityUrlKey() {
		return null;
	}

    /**
     * @return the key used in the soco to tell approval status.
     */
    public static String getSocoApprovedKey() {
		return null;
	}

    /**
     * @return the key used in the soco to tell approval status.
     */
    public static String getSocoFlaggedHiddenKey() {
		return null;
	}

    public static String getSocoDraftKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the approval status.
     */
    public static String getSchemaApprovedKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the approval status.
     */
    public static String getSchemaFlaggedHiddenKey() {
		return null;
	}

    public static String getSchemaIsDraftKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the parent id.
     */
    public static String getSchemaParentIdKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the provider id.
     */
    public static String getSchemaProviderIdKey() {
		return null;
	}

    /**
     * @return the key used in the schema to tell the creation time.
     */
    public static String getSchemaTimestampKey() {
		return null;
	}

    /**
     * @return the key used in schema to tell the attachment length.
     */
    public static String getSchemaAttachmentLengthKey() {
		return null;
	}

    /**
     * @return the key used in schema to tell the attachment creation time.
     */
    public static String getSchemaAttachmentUploadDateKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell the creation time.
     */
    public static String getSocoAddedKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell the parentid.
     */
    public static String getSocoParentIdKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell the tally timestamp.
     */
    public static String getSocoTallyTimestampKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell modification date.
     */
    public static String getSocoLastModifiedKey() {
		return null;
	}

    /**
     * @return the key used in soco to tell if there are attachments.
     */
    public static String getSchemaAttachmentKey() {
		return null;
	}

    /**
     * @return the key for a piece of ugc.
     */
    public static String getSocoKey() {
		return null;
	}

    /**
     * @return the key used in soco to give the description associated with the ugc.
     */
    public static String getSocoDescriptionKey() {
		return null;
	}

    /**
     * Escape things that need to go to solr.
     * @param base the original string
     * @return the converted string.
     */
    public static String escapeForSolr(final String base) {
		return null;
	}

    /**
     * @return The prefix used to identify non dynamic user date properties.
     */
    public static String getUserDatePrefix() {
		return null;
	}

    /**
     * @return The social base type.
     */
    public static String getSchemaBaseType() {
		return null;
	}

    /**
     * @return The CQ base type.
     */
    public static String getBaseType() {
		return null;
	}

    public static String getSchemaCQData() {
		return null;
	}

    /**
     * @return The CQ translation folder name.
     */
    public static String getTranslationFolderName() {
		return null;
	}

    /**
     * Convert Calendar/Date to millis for storage.
     * @param key Key to convert value for
     * @param asMap The map
     * @return milliseconds
     */
    private static Long toSchemaDate(final String key, final Map<String, Object> asMap) {
		return null;
	}

    /**
     * Convert Calendar/Date to millis for storage.
     * @param time Date/Calendar object
     * @return milliseconds
     */
    private static Long toSchemaDate(final Object time) {
		return null;
	}

    private static void spamFlagsToSchema(final Map<String, Object> cqMap, final Map<String, Object> asMap) {
	}

    private void setSolrId(final Map<String, Object> cqFormat, final String key, final Map<String, Object> asMap) {
	}

    /**
     * Map from the cq data format to the format expected by the schema.
     * @param cqFormat the incoming data. Note that this routine can change the contents of the map so that it is
     *            appropriate for caching. If you don't like that, make a copy of the map before sending it in.
     * @param key the key for the data
     * @return the mapped data
     */
    public Map<String, Object> toSchema(final Map<String, Object> cqFormat, final String key) {
		return null;
	}

    /**
     * Check all entries in Translation schema map to find the right key whose value + language matches strInputkey
     */
    private String getTranslationKeyEntry(final String strInputkey, final String strTranslationLanuage) {
		return null;
	}

    /**
     * Returns the translation language specified as the last name in the key path
     * @param strLanguage Language
     * @return String
     */
    public static String getMutliLingualLanguageKey(String strLanguage) {
		return null;
	}

    /**
     * Returns the translation language specified as the last name in the key
     * @param key language Key
     * @return String
     */
    private String getTranslationLanguageFromKey(final String key) {
		return null;
	}

    /**
     * This method first checks if translation schema map is empty or not, and then check if key provided has
     * translation as second last path, and returns true otherwise false
     * @param key
     */
    private boolean isTranslationProcessingRequired(final String key) {
		return false;
	}

    /**
     * Subclasses can implement this to muck with the mapping before it is actually committed. This is useful for the
     * little nitty differences between impls.
     * @param socoData the data in the SoCo format
     * @param schemaMap the data to be committed (in the common schema)
     */
    public abstract void finalizeSchemaMapping(Map<String, Object> socoData, Map<String, Object> schemaMap) {
	}

    /**
     * Format a Map using the keys expected by the schema.
     * @param data the data to be uploaded
     * @param key the key for the attachment
     * @return the data with the keys expected by the schema
     * @throws IOException on failure
     */
    public Map<String, Object> toAttachmentSchema(final Map<String, Object> data, final String key)
        throws IOException {
			return null;
		}

    private static void spamFlagsFromSchema(final Map<String, Object> cqMap, final Map<String, Object> asMap) {
	}

    /**
     * Return a Calendar object from a date value in the schema format. If value is an invalid date format (perhaps
     * from a legacy format we no longer know about or bugs from previous coding), return the current time as a place
     * holder.
     * @param key the key to convert
     * @param asDate the original date, in schema format
     * @return the converted date
     */
    public static Calendar fromSchemaDate(final String key, final Object asDate) {
		return null;
	}

    /**
     * Convert the data from schema into the format expected by SoCo.
     * @param schemaData the data to be converted
     * @return the mapped data
     */
    public Map<String, Object> fromSchema(final Map<String, Object> schemaData) {
		return null;
	}

    /**
     * Convert attachment data from the schema to what SoCo expects.
     * @param input the data in the schema
     * @return the data in the SoCo format
     */
    public Map<String, Object> fromAttachmentSchema(final Map<String, Object> input) {
		return null;
	}

    /**
     * Get the schema mapping of a SoCo field. Needed to map CQ fields in a query to Solr fields, since we don't have
     * information about the field type to be able to come up with the Solr name used when adding to index. Custom
     * fields have Solr schema names, so if not found, assume it is a custom dynamic field.
     * @param cqKey The CQ name of a field
     * @return The Solr index name for the CQ field or the input if not mapped (dynamic field name)
     */
    public String toSchemaKey(final String cqKey) {
		return null;
	}

    /**
     * Get the CQ mapping of a schema field. Needed to map schema key to CQ field. For facet fields etc, where we
     * don't have a map, just a string.
     * @param schemaKey The schema name of a field
     * @return The CQ field name or the input if not mapped (dynamic field name)
     */
    public String fromSchemaKey(final String schemaKey) {
		return null;
	}

    /**
     * Get the schema field name(s) associated with a given SoCo field.
     * @param mltField the SoCo field
     * @return the list of schema field names
     */
    public List<String> toSchemaKeys(final String mltField) {
		return null;
	}

    /**
     * Each Schema mapper class need to implement this method. Return list of verbatim supported
     * @return String[]
     */
    protected abstract String[] getASVerbatimList() {
		return null;
	}

    private List<String> mapToMultipleValues(final String mltField) {
		return null;
	}

    /**
     * Convert a lucene query to Solr query Maps the CQ names to Solr names. Handles the path wild card since Solr
     * does not find strings ending in a wildcard if it does not start with a wildcard.
     * @param luceneQuery the lucene query
     * @return the solr query
     */
    public String luceneToSolr(final String luceneQuery) {
		return null;
	}

    /**
     * Replace all occurrences of a string in a StringBuilder.
     * @param builder the StringBuilder (modified)
     * @param from source string
     * @param to replacement string
     */
    private static void replaceAll(final StringBuilder builder, final String from, final String to) {
	}

    /**
     * Convert to a format solr likes.
     * @param cqFormat the cq format data
     * @param key the key to convert
     * @return Map the converted data
     */
    public abstract Map<String, Object> toSolrSchema(final Map<String, Object> cqFormat, final String key) {
		return null;
	}

    /**
     * Checks if the 'key' is valid key that can be indexed in social-indexing engine (Solr,ElasticSearch,etc)
     * @param key Key
     * @return boolean
     */
    public boolean isIndexable(String key) {
		return false;
	}
}
