/***************************************************************************/
/*                                                                         */
/*                      ADOBE CONFIDENTIAL                                 */
/*                      _ _ _ _ _ _ _ _ _ _                                */
/*                                                                         */
/*  Copyright 2011, Adobe Systems Incorporated                             */
/*  All Rights Reserved.                                                   */
/*                                                                         */
/*  NOTICE: All information contained herein is, and remains the property  */
/*  of Adobe Systems Incorporated and its suppliers, if any. The           */
/*  intellectual and technical concepts contained herein are proprietary   */
/*  to Adobe Systems Incorporated and its suppliers and may be covered by  */
/*  U.S. and Foreign Patents, patents in process, and are protected by     */
/*  trade secret or copyright law.  Dissemination of this information or   */
/*  reproduction of this material is strictly forbidden unless prior       */
/*  written permission is obtained from Adobe Systems Incorporated.        */
/*                                                                         */
/***************************************************************************/

package com.adobe.octopus.extract;

import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.BundleException;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.Version;
import org.osgi.service.packageadmin.PackageAdmin;

/**
 * @author  <a href="mailto:tnaroska@adobe.com">tnaroska</a>
 * @version $Revision$
 * @since   1.0
 */
public final class BundleExtractor
{
	// --------------------------------------------------------------------------- Private Constants

	/** Root folder used for extracted resources in the bundle data store */
	private static final String ROOT = ".octopus";

	/** file that stores version information about this extractor's bundle and any of its 
	 * attached fragments */
	private static final String VERSION_FILE = ROOT + "/nativeCommExtractedVersions";
	
	// --------------------------------------------------------------------------- Private Variables

	/** Target bundle to operate on */
	private final Bundle bundle;
	
	/** Target bundle context to operate on */
	private final File baseDir;

	// -------------------------------------------------------------------------- Public Constructor

	/**
	 * Creates a BundleExtractor instance for the specified bundle.
	 *
	 * @param bundle - target bundle of this instance
	 */
	public BundleExtractor(Bundle bundle)
	{
		if (bundle == null)
		{
			throw new IllegalArgumentException("bundle == null");
		}

		this.bundle = bundle;

		BundleContext context = getBundleContext();

		baseDir = context.getDataFile(ROOT);
		if (baseDir == null)
		{
			throw new IllegalArgumentException("the OSGi platform has no file system support");
		}
		baseDir.mkdir();
		
		// Needed to retrieve fragment information
		PackageAdmin packageAdmin = (PackageAdmin)context.getService(context.getServiceReference(PackageAdmin.class.getName()));
	    
	    // Check if the versions have changed
	    try
		{
			if( resourcesNeedUpdate(packageAdmin) )
			{
				//If they have, delete any content of the working directory
				deleteDirectory( baseDir );
				baseDir.mkdir();
				//Create a new version file
				updateVersionFile(packageAdmin);
			}
		}
		catch (IOException e)
		{
			throw new IllegalStateException( e );
		}
	}

	/**
	 * Creates a BundleExtractor instance for the parent bundle of the specified class.
	 *
	 * @param clazz - a class object of the target bundle
	 */
	public BundleExtractor(Class<?> clazz)
	{
		this(FrameworkUtil.getBundle(clazz));
	}

	// ------------------------------------------------------------------------------ Public Methods

	/**
	 * Extracts a resource from the target bundle to the file system. If path denotes a folder
	 * all contents of the folder is extracted recursively.
	 *
	 * @param path - path to the bundle resource (always relative to the bundle root)
	 * @return file object pointing to the extracted file/folder
	 * @throws IOException - if path cannot be found in the bundle or extraction fails
	 */
	public File extractResource(String path) throws IOException
	{
		return null;
	}

	/**
	 * Extract the specified file from this bundle and mark it as executable (Linux only).
	 * 
	 * @param path - path to the executable bundle resource (always relative to the bundle root)
	 * @return file object pointing to the extracted executable file
	 * @throws IOException - if path cannot be found in the bundle or extraction fails
	 */
	public File extractExecutable(String path) throws IOException
	{
		return null;
	}

	// ----------------------------------------------------------------------------- Private Methods
	/**
	 * Recursively delete a directory
	 * @param path directory to delete
	 * @return see File.delete()
	 */
	private boolean deleteDirectory(File path) 
	{
		return false;
	}
	  
	/**
	 * Checks if the the version of this extractor's bundle or any of its attached fragments have changed.
	 * @return true if the any version has changed, false otherwise
	 * @throws IOException
	 */
	private boolean resourcesNeedUpdate(PackageAdmin packageAdmin) throws IOException
	{
		return false;
	}
	
	
	/**
	 * Updates/creates the version file with the ID and version info from the 
	 * extractor's bundle and any attached fragment bundles. 
	 * It will create the file if it does not exist
	 * @throws IOException
	 */
	private void updateVersionFile(PackageAdmin packageAdmin) throws IOException
	{
	}
	
	/**
	 * Get the current BundleContext
	 * @return the bundle's current context
	 */
	private BundleContext getBundleContext()
	{
		return null;
	}

	/**
	 * Find the specified path in the target bundle and resolve it to a java.io.File, unpacking
	 * bundle contents if necessary.
	 *
	 * @param path - bundle resource path (always relative to the bundle root)
	 * @return File object pointing to the extracted file.
	 * @throws IOException - if resource not found or cannot be resolved
	 */
	private File getEntry(String path) throws IOException
	{
		return null;
	}

	/**
	 * Copy out a bundle resource to the specified target location.
	 * If the resource already exists at the location it is not copied again.
	 * @param url - bundler resource URL
	 * @param target - target file location
	 * @throws IOException on error
	 */
	private void copyOut(final URL url, File target) throws IOException
	{
	}

	/**
	 * Copy inputstream to outputstream.
	 * @param is - source
	 * @param os - destination
	 * @return bytes copied
	 * @throws IOException on error
	 */
	private static long copy(InputStream is, OutputStream os) throws IOException
	{
		return 0L;
	}

	/**
	 * Mark the specified file as executable using Java File class.
	 * @param path - bundle resource path
	 * @param resolvedFile - actual file to modify
	 */
	private void markExecutable(String path, File resolvedFile) throws IOException
	{
	}

	/**
	 * Mark the specified file as executable spawning a chmod process.
	 * @param path - bundle resource path
	 * @param resolvedFile - actual file to modify
	 */
	private void markExecutable15(String path, File resolvedFile) throws IOException
	{
	}

	private static void safeClose(Closeable c)
	{
	}

	/**
	 * Try to start the target bundle if it is not already active.
	 */
	private void startBundle()
	{
	}
}
