/*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2012 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************/
package com.day.image;

import java.awt.Color;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.ByteLookupTable;
import java.awt.image.LookupOp;
import java.awt.image.LookupTable;
import java.awt.image.ShortLookupTable;

/**
 * The <code>MultitoneOp</code> class implements similar functionality as the
 * Photoshop <em>Duplex</em> function. It allows to specify one or more colors
 * and tonality curves for each color.
 * <p>
 * The {@link #filter} method expects to get a grayscale image to which the
 * colorization operation is applied.
 *
 * @version $Revision$, $Date$
 * @author fmeschbe
 * @since echidna
 * @audience wad
 */
public class MultitoneOp extends AbstractBufferedImageOp {

    /**
     * The {@link LookupTableHelper} used to create the lookup table for the
     * lookup operation in {@link #doFilter}.
     */
    private static final LookupTableHelper tableHelper;

    /**
     * The {@link ColorCurve} objects used to calculate color values for each
     * level of intensity.
     */
    private final ColorCurve[] colorCurves;

    /**
     * The maximum intensity value for all luminosities. This value is maximal
     * value of the sums of intensities of all color curves.
     */
    private final float maxAlpha;

    static {
        /**
         * Set up the lookup table helper. Use the Byte version for all operating
         * systems except linux, which has to use the Short version.
         */
        String os = System.getProperty("os.name", "-").toLowerCase();
        tableHelper = (os.indexOf("linux") >= 0)
                ? (LookupTableHelper) new ShortLookupTableHelper(null)
                : (LookupTableHelper) new ByteLookupTableHelper(null);
    }

    /**
     * Creates a <code>MultitoneOp</code> containing a standard color curve with
     * the single color. This creates a monotone result. This is a convenience
     * constructor which is the same as calling
     * {@link #MultitoneOp(Color[], RenderingHints)} with an array containing
     * the single color directly.
     *
     * @param color The color to use in the operation
     * @param hints the specified <code>RenderingHints</code>, or <code>null</code>
     */
    public MultitoneOp(Color color, RenderingHints hints) {
        this(new Color[]{ color }, hints);
    }

    /**
     * Creates a <code>MultitoneOp</code> containing a standard color curve with
     * more than one color. This creates a monotone result, where the color is
     * a mixture of the input colors.
     *
     * @param colors The colors to use in the operation
     * @param hints the specified <code>RenderingHints</code>, or <code>null</code>
     *
     * @throws NullPointerException if the colors array is <code>null</code> or
     *      if any of the color values is <code>null</code>.
     */
    public MultitoneOp(Color[] colors, RenderingHints hints) {
        super(hints);

        // check colors array
        if (colors == null) {
            throw new NullPointerException("colors");
        }

        colorCurves = new ColorCurve[colors.length];
        for (int i=0; i < colors.length; i++) {
            // check color
            if (colors[i] == null) {
                throw new NullPointerException("colors[" + i + "]");
            }

            colorCurves[i] = new ColorCurve(colors[i]);
        }

        maxAlpha = calculateMaxAlpha();
    }

    /**
     * Creates a <code>MultitoneOp</code> from the given {@link ColorCurve}
     * objects.
     *
     * @param colorCurves The {@link ColorCurve} objects from which to create
     *      this <code>MultitoneOp</code>
     * @param hints the specified <code>RenderingHints</code>, or <code>null</code>
     *
     * @throws NullPointerException if the colorCurves parameter or any of the
     *      entries is <code>null</code>.
     */
    public MultitoneOp(ColorCurve[] colorCurves, RenderingHints hints) {
        super(hints);

        // check color curve array
        if (colorCurves == null) {
            throw new NullPointerException("colorCurves");
        }

        this.colorCurves = new ColorCurve[colorCurves.length];
        for (int i=0; i < colorCurves.length; i++) {
            // check color curve
            if (colorCurves[i] == null) {
                throw new NullPointerException("colorCurves" + i + "]");
            }

            this.colorCurves[i] = new ColorCurve(colorCurves[i]);
        }

        maxAlpha = calculateMaxAlpha();
    }

    /**
     * Performs the multi tone operation. Note that the source image is expected
     * to be a gray scale image and the destination image must support color
     * images.
     * <p>
     * Note: This class supports filtering within an image, that is, calling
     * this method with <code>src == dst</code> is legal.
     *
     * @param src The src image to be filtered.
     * @param dst The dest image into which to place the resized image. This
     * 		may be <code>null</code> in which case a new image with the
     * 		correct size will be created.
     *
     * @return The newly created image (if dest was <code>null</code>) or dest
     * 		into which the resized src image has been drawn.
     *
     * @throws NullPointerException if the src image is <code>null</code>.
     */
    public BufferedImage filter(BufferedImage src, BufferedImage dst) {
		return null;
	}

    /**
     * Applies the {@link ColorCurve} objects to the grayscale source image and
     * stores the result in the destination image.
     *
     * @param src The source image
     * @param dst The destination image. This must not be <code>null</code>.
     */
    protected void doFilter(BufferedImage src, BufferedImage dst) {
	}

    private float calculateMaxAlpha() {
		return 0.0f;
	}

    //---------- internal interface/class to fix the ByteLookupTable bug #9432 -

    /**
     * The <code>LookupTableHelper</code> interface defines an interface which
     * will be implemented by imlementation specific helper data.
     * <p>
     * The issue of having this interface and companion classes is a bug in the
     * native lookup operation implementation on linux. To come around this bug
     * a ShortLookupTable is used on linux instead of a ByteLookupTable. The
     * drawback is that performance suffers when using the ShortLookupTable
     * because there is no native implementation for this operation.
     *
     * @version $Revision$, $Date$
     * @author fmeschbe
     * @since gumbaer
     * @audience core
     */
    private static interface LookupTableHelper {
        /**
         * Returns an instance of the implementation class.
         */
        public LookupTableHelper getInstance() {
			return null;
		}

        /**
         * Adds a mapping for the given color value.
         * @param index The index at which to set the mapping. Must be in the
         *      range 0..255.
         * @param red The color value for the red band
         * @param green The color value for the green band
         * @param blue The color value for the blue band
         * @param alpha The color value for the alpha band
         * @throws IndexOutOfBoundsException if the index is outside of the
         *      range 0..255.
         */
        public void addMapping(int index, float red, float green, float blue, float alpha) {
		}

        /**
         * Returns a new lookup table created from the mappings defined with the
         * {@link #addMapping} method. Each call to this method returns a new
         * instance of <code>LookupTable</code>.
         */
        public LookupTable getLookupTable() {
			return null;
		}
    }

    /**
     * The <code>ByteLookupTableHelper</code> class implements the
     * {@link LookupTableHelper} interface to provide a <code>ByteLookupTable</code>.
     *
     * @version $Revision$, $Date$
     * @author fmeschbe
     * @since gumbaer
     * @audience core
     */
    private static class ByteLookupTableHelper implements LookupTableHelper {

        /** The table data */
        private final byte[][] table;

        private ByteLookupTableHelper(byte[][] table) {
            this.table = table;
        }

        public LookupTableHelper getInstance() {
			return null;
		}

        public void addMapping(int index, float red, float green, float blue, float alpha) {
		}

        public LookupTable getLookupTable() {
			return null;
		}
    }

    /**
     * The <code>ShortLookupTableHelper</code> class implements the
     * {@link LookupTableHelper} interface to provide a <code>ShortLookupTable</code>.
     *
     * @version $Revision$, $Date$
     * @author fmeschbe
     * @since gumbaer
     * @audience core
     */
    private static class ShortLookupTableHelper implements LookupTableHelper {

        /** The table data */
        private final short[][] table;

        private ShortLookupTableHelper(short[][] table) {
            this.table = table;
        }

        public LookupTableHelper getInstance() {
			return null;
		}

        public void addMapping(int index, float red, float green, float blue, float alpha) {
		}

        public LookupTable getLookupTable() {
			return null;
		}
    }
}