/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.oak.query;

import static com.google.common.base.Preconditions.checkNotNull;
import static com.google.common.collect.Maps.newHashMap;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;

import org.apache.jackrabbit.oak.api.PropertyValue;
import org.apache.jackrabbit.oak.api.QueryEngine;
import org.apache.jackrabbit.oak.api.Type;
import org.apache.jackrabbit.oak.commons.PathUtils;
import org.apache.jackrabbit.oak.namepath.NamePathMapper;
import org.apache.jackrabbit.oak.query.QueryOptions.Traversal;
import org.apache.jackrabbit.oak.query.ast.AstElementFactory;
import org.apache.jackrabbit.oak.query.ast.BindVariableValueImpl;
import org.apache.jackrabbit.oak.query.ast.ColumnImpl;
import org.apache.jackrabbit.oak.query.ast.ConstraintImpl;
import org.apache.jackrabbit.oak.query.ast.DynamicOperandImpl;
import org.apache.jackrabbit.oak.query.ast.JoinConditionImpl;
import org.apache.jackrabbit.oak.query.ast.JoinType;
import org.apache.jackrabbit.oak.query.ast.LiteralImpl;
import org.apache.jackrabbit.oak.query.ast.NodeTypeInfo;
import org.apache.jackrabbit.oak.query.ast.NodeTypeInfoProvider;
import org.apache.jackrabbit.oak.query.ast.Operator;
import org.apache.jackrabbit.oak.query.ast.OrderingImpl;
import org.apache.jackrabbit.oak.query.ast.PropertyExistenceImpl;
import org.apache.jackrabbit.oak.query.ast.PropertyInexistenceImpl;
import org.apache.jackrabbit.oak.query.ast.PropertyValueImpl;
import org.apache.jackrabbit.oak.query.ast.SelectorImpl;
import org.apache.jackrabbit.oak.query.ast.SourceImpl;
import org.apache.jackrabbit.oak.query.ast.StaticOperandImpl;
import org.apache.jackrabbit.oak.spi.query.PropertyValues;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The SQL2 parser can convert a JCR-SQL2 query to a query. The 'old' SQL query
 * language (here named SQL-1) is also supported.
 */
public class SQL2Parser {
    
    private static final Logger LOG = LoggerFactory.getLogger(SQL2Parser.class);

    // Character types, used during the tokenizer phase
    private static final int CHAR_END = -1, CHAR_IGNORE = 0;
    private static final int CHAR_VALUE = 2, CHAR_QUOTED = 3;
    private static final int CHAR_NAME = 4, CHAR_SPECIAL_1 = 5, CHAR_SPECIAL_2 = 6;
    private static final int CHAR_STRING = 7, CHAR_DECIMAL = 8, CHAR_BRACKETED = 9;

    // Token types
    private static final int KEYWORD = 1, IDENTIFIER = 2, PARAMETER = 3, END = 4, VALUE = 5;
    private static final int MINUS = 12, PLUS = 13, OPEN = 14, CLOSE = 15;

    private final NodeTypeInfoProvider nodeTypes;

    // The query as an array of characters and character types
    private String statement;
    private char[] statementChars;
    private int[] characterTypes;

    // The current state of the parser
    private int parseIndex;
    private int currentTokenType;
    private String currentToken;
    private boolean currentTokenQuoted;
    private PropertyValue currentValue;
    private ArrayList<String> expected;

    // The bind variables
    private HashMap<String, BindVariableValueImpl> bindVariables;

    // The list of selectors of this query
    private final Map<String, SelectorImpl> selectors = newHashMap();

    // SQL injection protection: if disabled, literals are not allowed
    private boolean allowTextLiterals = true;
    private boolean allowNumberLiterals = true;
    private boolean includeSelectorNameInWildcardColumns = true;

    private final AstElementFactory factory = new AstElementFactory();

    private boolean supportSQL1;

    private NamePathMapper namePathMapper;
    
    private final QueryEngineSettings settings;
    
    private boolean literalUsageLogged;

    /**
     * Create a new parser. A parser can be re-used, but it is not thread safe.
     * 
     * @param namePathMapper the name-path mapper to use
     * @param nodeTypes the nodetypes
     * @param settings the query engine settings
     */
    public SQL2Parser(NamePathMapper namePathMapper, NodeTypeInfoProvider nodeTypes, QueryEngineSettings settings) {
        this.namePathMapper = namePathMapper;
        this.nodeTypes = checkNotNull(nodeTypes);
        this.settings = checkNotNull(settings);
    }

    /**
     * Parse the statement and return the query.
     *
     * @param query the query string
     * @param initialise if performing the query init ({@code true}) or not ({@code false})
     * @return the query
     * @throws ParseException if parsing fails
     */
    public Query parse(final String query, final boolean initialise) throws ParseException {
		return null;
	}
    
    /**
     * as {@link #parse(String, boolean)} by providing {@code true} to the initialisation flag.
     * 
     * @param query
     * @return the parsed query
     * @throws ParseException
     */
    public Query parse(final String query) throws ParseException {
		return null;
	}
    
    private QueryImpl parseSelect() throws ParseException {
		return null;
	}

    private static void addColumnIfNecessary(ArrayList<ColumnOrWildcard> list,
            String columnName, String propertyName) {
			}

    /**
     * Enable or disable support for SQL-1 queries.
     *
     * @param sql1 the new value
     */
    public void setSupportSQL1(boolean sql1) {
	}

    private SelectorImpl parseSelector() throws ParseException {
		return null;
	}

    private String readName() throws ParseException {
		return null;
	}

    private SourceImpl parseSource() throws ParseException {
		return null;
	}

    private JoinConditionImpl parseJoinCondition() throws ParseException {
		return null;
	}

    private ConstraintImpl parseConstraint() throws ParseException {
		return null;
	}

    private ConstraintImpl parseAnd() throws ParseException {
		return null;
	}

    private ConstraintImpl parseCondition() throws ParseException {
		return null;
	}

    private ConstraintImpl parseCondition(DynamicOperandImpl left) throws ParseException {
		return null;
	}

    private PropertyExistenceImpl getPropertyExistence(PropertyValueImpl p) throws ParseException {
		return null;
	}
    
    private PropertyInexistenceImpl getPropertyInexistence(PropertyValueImpl p) throws ParseException {
		return null;
	}

    private ConstraintImpl parseConditionFunctionIf(String functionName) throws ParseException {
		return null;
	}

    private String readAbsolutePath() throws ParseException {
		return null;
	}

    private String readPath() throws ParseException {
		return null;
	}

    private DynamicOperandImpl parseDynamicOperand() throws ParseException {
		return null;
	}

    private DynamicOperandImpl parseExpressionFunction(String functionName) throws ParseException {
		return null;
	}

    private PropertyValueImpl parsePropertyValue(String name) throws ParseException {
		return null;
	}

    private StaticOperandImpl parseStaticOperand() throws ParseException {
		return null;
	}

    /**
     * Create a literal from a parsed value.
     *
     * @param value the original value
     * @return the literal
     */
    private LiteralImpl getUncastLiteral(PropertyValue value) {
		return null;
	}

    private PropertyValue parseCastAs(PropertyValue value)
            throws ParseException {
				return null;
			}

    /**
     * Get the property type from the given case insensitive name.
     *
     * @param name the property type name (case insensitive)
     * @return the type, or {@code PropertyType.UNDEFINED} if unknown
     */
    public static int getPropertyTypeFromName(String name) {
		return 0;
	}

    private static boolean matchesPropertyType(int propertyType, String name) {
		return false;
	}

    private OrderingImpl[] parseOrder() throws ParseException {
		return null;
	}

    private ArrayList<ColumnOrWildcard> parseColumns() throws ParseException {
		return null;
	}
    
    private boolean readOptionalAlias(ColumnOrWildcard column) throws ParseException {
		return false;
	}

    private ColumnImpl[] resolveColumns(ArrayList<ColumnOrWildcard> list) throws ParseException {
		return null;
	}

    private void addWildcardColumns(
            Collection<ColumnImpl> columns, String selectorName)
            throws ParseException {
			}

    private void addWildcardColumns(
            Collection<ColumnImpl> columns, SelectorImpl selector) {
			}

    private boolean readIf(String token) throws ParseException {
		return false;
	}

    private boolean isToken(String token) {
		return false;
	}

    private void read(String expected) throws ParseException {
	}

    private PropertyValue readString() throws ParseException {
		return null;
	}

    private void addExpected(String token) {
	}

    private void initialize(String query) throws ParseException {
	}

    private void checkRunOver(int i, int len, int startLoop) throws ParseException {
	}

    private void read() throws ParseException {
	}

    private void readString(int i, char end) throws ParseException {
	}

    private void checkLiterals(boolean text) throws ParseException {
	}

    private void readDecimal(int start, int i) throws ParseException {
	}

    private ParseException getSyntaxError() {
		return null;
	}

    private ParseException getSyntaxError(String expected) {
		return null;
	}

    /**
     * Represents a column or a wildcard in a SQL expression.
     * This class is temporarily used during parsing.
     */
    static class ColumnOrWildcard {
        String selectorName;
        String propertyName;
        String columnName;
    }

    /**
     * Get the selector name if only one selector exists in the query.
     * If more than one selector exists, an exception is thrown.
     *
     * @return the selector name
     */
    private String getOnlySelectorName() throws ParseException {
		return null;
	}

    public static String escapeStringLiteral(String value) {
		return null;
	}

    /**
     * Enable or disable support for text literals in queries. The default is enabled.
     *
     * @param allowTextLiterals
     */
    public void setAllowTextLiterals(boolean allowTextLiterals) {
	}

    public void setAllowNumberLiterals(boolean allowNumberLiterals) {
	}

    public void setIncludeSelectorNameInWildcardColumns(boolean value) {
	}

    /**
     * Whether the given statement is an internal query.
     *  
     * @param statement the statement
     * @return true for an internal query
     */
    public static boolean isInternal(String statement) {
		return false;
	}

}
