/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.jackrabbit.spi.commons.value;

import org.apache.jackrabbit.spi.QValue;
import org.apache.jackrabbit.spi.Name;
import org.apache.jackrabbit.spi.Path;
import org.apache.jackrabbit.util.ISO8601;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;
import javax.jcr.Binary;

import java.util.Calendar;
import java.util.TimeZone;
import java.math.BigDecimal;
import java.net.URI;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Serializable;
import java.io.StringWriter;
import java.io.Writer;

/**
 * <code>AbstractQValue</code>...
 */
public abstract class AbstractQValue implements QValue, Serializable {

    private static final long serialVersionUID = 6976433831974695272L;

    protected final Object val;
    protected final int type;

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value The value.
     * @param type The property type.
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Object value, int type) {
        if (value == null) {
            throw new IllegalArgumentException("null value");
        }
        this.val = value;
        this.type = type;
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @param type
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code> or if the <code>type</code> is neither STRING nor
     * REFERENCE/WEAKREFERENCE.
     */
    protected AbstractQValue(String value, int type) {
        if (value == null) {
            throw new IllegalArgumentException("null value");
        }
        if (!(type == PropertyType.STRING
                || type == PropertyType.DATE // JCR-3083
                || type == PropertyType.REFERENCE
                || type == PropertyType.WEAKREFERENCE)) {
            throw new IllegalArgumentException();
        }
        this.val = value;
        this.type = type;
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Long value) {
        this(value, PropertyType.LONG);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Double value) {
        this(value, PropertyType.DOUBLE);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Boolean value) {
        this(value, PropertyType.BOOLEAN);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Calendar value) {
        val = ISO8601.format(value);
        type = PropertyType.DATE;
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Name value) {
        this(value, PropertyType.NAME);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(Path value) {
        this(value, PropertyType.PATH);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(BigDecimal value) {
        this(value, PropertyType.DECIMAL);
    }

    /**
     * Create a new <code>AbstractQValue</code>.
     *
     * @param value
     * @throws IllegalArgumentException if the passed <code>value</code>
     * is <code>null</code>.
     */
    protected AbstractQValue(URI value) {
        this(value, PropertyType.URI);
    }

    //---------------------------------------------------------< QValue >---
    /**
     * @see QValue#getType()
     */
    public int getType() {
		return 0;
	}

    /**
     * @see QValue#getLength()
     */
    public long getLength() throws RepositoryException {
		return 0L;
	}

    /**
     * @see QValue#getName()
     */
    public Name getName() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#getCalendar()
     */
    public Calendar getCalendar() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#getDecimal()
     */
    public BigDecimal getDecimal() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#getURI()
     */
    public URI getURI() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#getDouble()
     */
    public double getDouble() throws RepositoryException {
		return 0.0d;
	}

    /**
     * @see QValue#getLong()
     */
    public long getLong() throws RepositoryException {
		return 0L;
	}

    /**
     * @throws RepositoryException
     * @see QValue#getBoolean()
     */
    public boolean getBoolean() throws RepositoryException {
		return false;
	}

    /**
     * @see QValue#getPath()
     */
    public Path getPath() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#getPath()
     */
    public String getString() throws RepositoryException {
		return null;
	}

    /**
     * This implementation creates a binary instance that uses
     * {@link #getStream()} and skipping on the given stream as its underlying
     * mechanism to provide random access defined on {@link Binary}.
     *
     * @see QValue#getBinary()
     */
    public Binary getBinary() throws RepositoryException {
		return null;
	}

    /**
     * @see QValue#discard()
     */
    public void discard() {
	}

    //-------------------------------------------------------------< Object >---
    /**
     * Returns the string representation of this internal value.
     *
     * @return string representation of this internal value
     */
    @Override
    public String toString() {
		return null;
	}

    /**
     * Default implementation of the equals method. Subclasses may optimize
     * this e.g. by special handling for DATE properties.
     *
     * @param obj
     * @see Object#equals(Object)
     */
    @Override
    public boolean equals(Object obj) {
		return false;
	}

    /**
     * Default calculation of the hashCode. Subclasses may optimize
     * this e.g. by special handling for DATE properties.
     *
     * @return the hashCode of the internal value object.
     * @see Object#hashCode()
     */
    @Override
    public int hashCode() {
		return 0;
	}
}
