/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.base.connectors.announcement;

import java.io.StringReader;
import java.io.StringWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.json.Json;
import javax.json.JsonArray;
import javax.json.JsonArrayBuilder;
import javax.json.JsonObject;
import javax.json.JsonObjectBuilder;
import javax.json.JsonReaderFactory;
import javax.json.JsonValue;
import javax.json.JsonValue.ValueType;

import org.apache.sling.api.resource.ModifiableValueMap;
import org.apache.sling.api.resource.PersistenceException;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.discovery.ClusterView;
import org.apache.sling.discovery.InstanceDescription;
import org.apache.sling.discovery.commons.providers.DefaultClusterView;
import org.apache.sling.discovery.commons.providers.DefaultInstanceDescription;
import org.apache.sling.discovery.commons.providers.NonLocalInstanceDescription;

/**
 * An announcement is the information exchanged by the topology connector and
 * contains all clusters and instances which both the topology connector client
 * and servlet see (in their part before joining the two worlds).
 * <p>
 * An announcement is exchanged in json format and carries a timeout.
 */
public class Announcement {

    /** the protocol version this announcement currently represents. Mismatching protocol versions are
     * used to detect incompatible topology connectors
     */
    private final static int PROTOCOL_VERSION = 1;

    /** the sling id of the owner of this announcement. the owner is where this announcement comes from **/
    private final String ownerId;

    /** announcement protocol version **/
    private final int protocolVersion;

    /** the local cluster view **/
    private ClusterView localCluster;

    /** the incoming instances **/
    private List<Announcement> incomings = new LinkedList<Announcement>();

    /** whether or not this annoucement was inherited (response of a connect) or incoming (the connect) **/
    private boolean inherited = false;

    /** some information about the server where this announcement came from **/
    private String serverInfo;

    /** whether or not this announcement represents a loop detected in the topology connectors **/
    private boolean loop = false;

    /** SLING-3382: Sets the backoffInterval which the connector servlets passes back to the client to use as the next heartbeatInterval **/
    private long backoffInterval = -1;

    /** SLING-3382: the resetBackoff flag is sent from client to server and indicates that the client wants to start from (backoff) scratch **/
    private boolean resetBackoff = false;
    
    private long originallyCreatedAt = -1;
    
    private long receivedAt = System.currentTimeMillis();
    
    private static final JsonReaderFactory jsonReaderFactory;
    static {
        Map<String, Object> config = new HashMap<String, Object>();
        config.put("org.apache.johnzon.supports-comments", true);
        jsonReaderFactory = Json.createReaderFactory(config);
    }

    public Announcement(final String ownerId) {
        this(ownerId, PROTOCOL_VERSION);
    }

    public Announcement(final String ownerId, int protocolVersion) {
        if (ownerId==null || ownerId.length()==0) {
            throw new IllegalArgumentException("ownerId must not be null or empty");
        }
        this.ownerId = ownerId;
        this.protocolVersion = protocolVersion;
    }

    @Override
    public String toString() {
		return null;
	}

    /** check whether this is announcement contains the valid protocol version **/
    public boolean isCorrectVersion() {
		return false;
	}

    /** check whether this is a valid announcement, containing the minimal information **/
    public boolean isValid() {
		return false;
	}

    /** set the inherited flag - if true this means this announcement is the response of a topology connect **/
    public void setInherited(final boolean inherited) {
	}

    /** Returns the inherited flag - if true this means that this announcement is the response of a topology connect **/
    public boolean isInherited() {
		return false;
	}

    /** Sets the loop falg - set true when this announcement should represent a loop detected in the topology connectors **/
    public void setLoop(final boolean loop) {
	}
    
    /** Sets the backoffInterval which the connector servlets passes back to the client to use as the next heartbeatInterval **/
    public void setBackoffInterval(long backoffInterval) {
	}
    
    /** Gets the backoffInterval which the connector servlets passes back to the client to use as the next heartbeatInterval **/
    public long getBackoffInterval() {
		return 0L;
	}
    
    public long getOriginallyCreatedAt() {
		return 0L;
	}
    
    public long getReceivedAt() {
		return 0L;
	}
    
    /** sets the resetBackoff flag **/
    public void setResetBackoff(boolean resetBackoff) {
	}
    
    /** gets the resetBackoff flag **/
    public boolean getResetBackoff() {
		return false;
	}

    /** Returns the loop flag - set when this announcement represents a loop detected in the topology connectors **/
    public boolean isLoop() {
		return false;
	}

    /** Returns the protocolVersion of this announcement **/
    public int getProtocolVersion() {
		return 0;
	}

    /** sets the information about the server where this announcement came from **/
    public void setServerInfo(final String serverInfo) {
	}

    /** the information about the server where this announcement came from **/
    public String getServerInfo() {
		return null;
	}

    /**
     * Returns the slingid of the owner of this announcement.
     * <p>
     * The owner is the instance which initiated the topology connection
     */
    public String getOwnerId() {
		return null;
	}
    
    /** Convert this announcement into a json object **/
    private JsonObject asJSONObject(boolean filterTimes) {
		return null;
	}

    /** Create an announcement form json **/
    public static Announcement fromJSON(final String topologyAnnouncementJSON) {
		return null;
	}

    /** create a clusterview from json **/
    private static ClusterView asClusterView(final String localClusterViewJSON) {
		return null;
	}

    /** convert a clusterview into json **/
    private static JsonObject asJSON(final ClusterView clusterView) {
		return null;
	}

    /** create an instancedescription from json **/
    private static DefaultInstanceDescription asInstance(final JsonObject anInstance) {
		return null;
	}

    /** convert an instance description into a json object **/
    private static JsonObject asJSON(final InstanceDescription instanceDescription) {
		return null;
	}

    /** sets the local clusterview **/
    public void setLocalCluster(ClusterView localCluster) {
	}

    /** adds an incoming announcement to this announcement **/
    public void addIncomingTopologyAnnouncement(
            Announcement incomingTopologyAnnouncement) {
			}

    /** Convert this announcement into json **/
    public String asJSON() {
		return null;
	}
    
    private static String asJSON(JsonValue json) {
		return null;
	}

    /** the key which is unique to this announcement **/
    public String getPrimaryKey() {
		return null;
	}

    /** Returns the list of instances that are contained in this announcement **/
    public Collection<InstanceDescription> listInstances() {
		return null;
	}

    /**
     * Persists this announcement using the given 'announcements' resource,
     * under which a node with the primary key is created
     **/
    public void persistTo(Resource announcementsResource)
            throws PersistenceException {
			}

	/**
     * Remove all announcements that match the given owner Id
     */
    public void removeInherited(final String ownerId) {
	}

    /**
     * Compare this Announcement with another one, ignoring the 'created'
     * property - which gets added to the JSON object automatically due
     * to SLING-3389 wire-backwards-compatibility - and backoffInterval
     * introduced as part of SLING-3382
     */
    public boolean correspondsTo(Announcement announcement) {
		return false;
	}

    public void registerPing(Announcement incomingAnnouncement) {
	}

}
