/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.sling.discovery.commons.providers.spi.base;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Base class which implements the concept of a 'BackgroundCheck',
 * a thread that periodically executes a check until that one succeeds.
 * <p>
 */
public abstract class AbstractServiceWithBackgroundCheck {

    protected final Logger logger = LoggerFactory.getLogger(getClass());

    protected String slingId;

    /**
     * The BackgroundCheckRunnable implements the details of
     * calling BackgroundCheck.check and looping until it 
     * returns true
     */
    final class BackgroundCheckRunnable implements Runnable {
        private final Runnable callback;
        private final BackgroundCheck check;
        private final long timeoutMillis;
        private volatile boolean cancelled;
        private final String threadName;
        
        // for testing only:
        private final Object waitObj = new Object();
        private int waitCnt;
        private volatile boolean done;
        private long waitInterval;

        private BackgroundCheckRunnable(Runnable callback, 
                BackgroundCheck check, long timeoutMillis, long waitInterval, String threadName) {
            this.callback = callback;
            this.check = check;
            this.timeoutMillis = timeoutMillis;
            if (waitInterval <= 0) {
                throw new IllegalArgumentException("waitInterval must be greater than 0: "+waitInterval);
            }
            this.waitInterval = waitInterval;
            this.threadName = threadName;
        }
        
        boolean isDone() {
			return false;
		}

        @Override
        public void run() {
		}
        
        boolean cancelled() {
			return false;
		}

        void cancel() {
		}

        public void triggerCheck() {
		}
    }

    /**
     * A BackgroundCheck is anything that can be periodically
     * checked until it eventually returns true.
     */
    interface BackgroundCheck {
        
        boolean check() {
			return false;
		}
        
    }
    
    protected BackgroundCheckRunnable backgroundCheckRunnable;
    
    /**
     * Cancel the currently ongoing background check if
     * there is any ongoing.
     */
    protected void cancelPreviousBackgroundCheck() {
	}
    
    /**
     * Start a new BackgroundCheck in a separate thread, that
     * periodically calls BackgroundCheck.check and upon completion
     * calls the provided callback.run()
     * @param threadName the name of the thread (to allow identifying the thread)
     * @param check the BackgroundCheck to periodically invoke with check()
     * @param callback the Runnable to invoke upon a successful check()
     * @param timeoutMillis a timeout at which point the BackgroundCheck is
     * terminated and no callback is invoked. Note that this happens unnoticed
     * at the moment, ie there is no feedback about whether a background
     * check was successfully termianted (ie callback was invoked) or
     * whether the timeout has hit (that's left as a TODO if needed).
     */
    protected void startBackgroundCheck(String threadName, final BackgroundCheck check, final Runnable callback, final long timeoutMillis, final long waitMillis) {
	}
    

    /** for testing only! **/
    protected void triggerBackgroundCheck() {
	}
}
